#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include "types.h"
#include "const.h"
#include "random.h"

uint m_z;
uint m_w;
int seeded;
int buffered;
double buffer;

uint GetUint()
{
	if(seeded == 0)
	{
		SetRandomSeed();
	}

    m_z = 36969 * (m_z & 65535) + (m_z >> 16);
    m_w = 18000 * (m_w & 65535) + (m_w >> 16);
    return (m_z << 16) + m_w;
}

double Random()
{
    // 0 <= u < 2^32
    uint u = GetUint();
    // The magic number below is 1/(2^32 + 2).
    // The result is strictly between 0 and 1.
    return (u + 1.0) * 2.328306435454494e-10;
}

double Uniform(double min, double max)
{
	double r = Random();
	return r * (max - min) + min;
}

int Discrete(int include_min, int exclude_max)
{
	double r = Random();
	int offset = (int)floor(r * ((double)(exclude_max - include_min) - EPSILON));
	return offset + include_min;
}

double Gaussian(double sigma, double mu)
{
	if(buffered == TRUE)
	{
		buffered = FALSE;
		return mu + sigma * buffer;
	}

	double s, u, v;

	do
	{
		u = Uniform(-1.0, 1.0);
		v = Uniform(-1.0, 1.0);
		s = u * u + v * v;
	} while(s == 0.0 || s >= 1.0);

	double a = sqrt(-2.0 * log(s) / s);
	double z = u * a;
	buffer = v * a;
	buffered = TRUE;
	return mu + sigma * z;
}

double Exponential(double min, double decay)
{
	double r = Random();
	return min - log(r) / decay;
}

void SetSpecificSeed(uint seed)
{
	seeded = 1;
	m_z = seed % 4294967296;
	m_w = seed >> 16;
}

void SetRandomSeed()
{
	uint iseed = (uint)time(NULL);
	SetSpecificSeed(iseed);
}